# File for configuration of the stockdex package

BASE_URL = "https://query2.finance.yahoo.com/v8/finance"
FUNDAMENTALS_BASE_URL = (
    "https://query2.finance.yahoo.com/ws/fundamentals-timeseries/v1/finance/timeseries"
)

QUARTERLY_INCOME_STATEMENT_COLUMNS = [
    "quarterlyTaxEffectOfUnusualItems",
    "quarterlyTaxRateForCalcs",
    "quarterlyNormalizedEBITDA",
    "quarterlyNormalizedDilutedEPS",
    "quarterlyNormalizedBasicEPS",
    "quarterlyTotalUnusualItems",
    "quarterlyTotalUnusualItemsExcludingGoodwill",
    "quarterlyNetIncomeFromContinuingOperationNetMinorityInterest",
    "quarterlyReconciledDepreciation",
    "quarterlyReconciledCostOfRevenue",
    "quarterlyEBITDA",
    "quarterlyEBIT",
    "quarterlyNetInterestIncome",
    "quarterlyInterestExpense",
    "quarterlyInterestIncome",
    "quarterlyContinuingAndDiscontinuedDilutedEPS",
    "quarterlyContinuingAndDiscontinuedBasicEPS",
    "quarterlyNormalizedIncome",
    "quarterlyNetIncomeFromContinuingAndDiscontinuedOperation",
    "quarterlyTotalExpenses",
    "quarterlyRentExpenseSupplemental",
    "quarterlyReportedNormalizedDilutedEPS",
    "quarterlyReportedNormalizedBasicEPS",
    "quarterlyTotalOperatingIncomeAsReported",
    "quarterlyDividendPerShare",
    "quarterlyDilutedAverageShares",
    "quarterlyBasicAverageShares",
    "quarterlyDilutedEPS",
    "quarterlyDilutedEPSOtherGainsLosses",
    "quarterlyTaxLossCarryforwardDilutedEPS",
    "quarterlyDilutedAccountingChange",
    "quarterlyDilutedExtraordinary",
    "quarterlyDilutedDiscontinuousOperations",
    "quarterlyDilutedContinuousOperations",
    "quarterlyBasicEPS",
    "quarterlyBasicEPSOtherGainsLosses",
    "quarterlyTaxLossCarryforwardBasicEPS",
    "quarterlyBasicAccountingChange",
    "quarterlyBasicExtraordinary",
    "quarterlyBasicDiscontinuousOperations",
    "quarterlyBasicContinuousOperations",
    "quarterlyDilutedNIAvailtoComStockholders",
    "quarterlyAverageDilutionEarnings",
    "quarterlyNetIncomeCommonStockholders",
    "quarterlyOtherunderPreferredStockDividend",
    "quarterlyPreferredStockDividends",
    "quarterlyNetIncome",
    "quarterlyMinorityInterests",
    "quarterlyNetIncomeIncludingNoncontrollingInterests",
    "quarterlyNetIncomeFromTaxLossCarryforward",
    "quarterlyNetIncomeExtraordinary",
    "quarterlyNetIncomeDiscontinuousOperations",
    "quarterlyNetIncomeContinuousOperations",
    "quarterlyEarningsFromEquityInterestNetOfTax",
    "quarterlyTaxProvision",
    "quarterlyPretaxIncome",
    "quarterlyOtherIncomeExpense",
    "quarterlyOtherNonOperatingIncomeExpenses",
    "quarterlySpecialIncomeCharges",
    "quarterlyGainOnSaleOfPPE",
    "quarterlyGainOnSaleOfBusiness",
    "quarterlyOtherSpecialCharges",
    "quarterlyWriteOff",
    "quarterlyImpairmentOfCapitalAssets",
    "quarterlyRestructuringAndMergernAcquisition",
    "quarterlySecuritiesAmortization",
    "quarterlyEarningsFromEquityInterest",
    "quarterlyGainOnSaleOfSecurity",
    "quarterlyNetNonOperatingInterestIncomeExpense",
    "quarterlyTotalOtherFinanceCost",
    "quarterlyInterestExpenseNonOperating",
    "quarterlyInterestIncomeNonOperating",
    "quarterlyOperatingIncome",
    "quarterlyOperatingExpense",
    "quarterlyOtherOperatingExpenses",
    "quarterlyOtherTaxes",
    "quarterlyProvisionForDoubtfulAccounts",
    "quarterlyDepreciationAmortizationDepletionIncomeStatement",
    "quarterlyDepletionIncomeStatement",
    "quarterlyDepreciationAndAmortizationInIncomeStatement",
    "quarterlyAmortization",
    "quarterlyAmortizationOfIntangiblesIncomeStatement",
    "quarterlyDepreciationIncomeStatement",
    "quarterlyResearchAndDevelopment",
    "quarterlySellingGeneralAndAdministration",
    "quarterlySellingAndMarketingExpense",
    "quarterlyGeneralAndAdministrativeExpense",
    "quarterlyOtherGandA",
    "quarterlyInsuranceAndClaims",
    "quarterlyRentAndLandingFees",
    "quarterlySalariesAndWages",
    "quarterlyGrossProfit",
    "quarterlyCostOfRevenue",
    "quarterlyTotalRevenue",
    "quarterlyExciseTaxes",
    "quarterlyOperatingRevenue",
]

ANNUAL_INCOME_STATEMENT_COLUMNS = [
    "annualTaxEffectOfUnusualItems",
    "annualTaxRateForCalcs",
    "annualNormalizedEBITDA",
    "annualNormalizedDilutedEPS",
    "annualNormalizedBasicEPS",
    "annualTotalUnusualItems",
    "annualTotalUnusualItemsExcludingGoodwill",
    "annualNetIncomeFromContinuingOperationNetMinorityInterest",
    "annualReconciledDepreciation",
    "annualReconciledCostOfRevenue",
    "annualEBITDA",
    "annualEBIT",
    "annualNetInterestIncome",
    "annualInterestExpense",
    "annualInterestIncome",
    "annualContinuingAndDiscontinuedDilutedEPS",
    "annualContinuingAndDiscontinuedBasicEPS",
    "annualNormalizedIncome",
    "annualNetIncomeFromContinuingAndDiscontinuedOperation",
    "annualTotalExpenses",
    "annualRentExpenseSupplemental",
    "annualReportedNormalizedDilutedEPS",
    "annualReportedNormalizedBasicEPS",
    "annualTotalOperatingIncomeAsReported",
    "annualDividendPerShare",
    "annualDilutedAverageShares",
    "annualBasicAverageShares",
    "annualDilutedEPS",
    "annualDilutedEPSOtherGainsLosses",
    "annualTaxLossCarryforwardDilutedEPS",
    "annualDilutedAccountingChange",
    "annualDilutedExtraordinary",
    "annualDilutedDiscontinuousOperations",
    "annualDilutedContinuousOperations",
    "annualBasicEPS",
    "annualBasicEPSOtherGainsLosses",
    "annualTaxLossCarryforwardBasicEPS",
    "annualBasicAccountingChange",
    "annualBasicExtraordinary",
    "annualBasicDiscontinuousOperations",
    "annualBasicContinuousOperations",
    "annualDilutedNIAvailtoComStockholders",
    "annualAverageDilutionEarnings",
    "annualNetIncomeCommonStockholders",
    "annualOtherunderPreferredStockDividend",
    "annualPreferredStockDividends",
    "annualNetIncome",
    "annualMinorityInterests",
    "annualNetIncomeIncludingNoncontrollingInterests",
    "annualNetIncomeFromTaxLossCarryforward",
    "annualNetIncomeExtraordinary",
    "annualNetIncomeDiscontinuousOperations",
    "annualNetIncomeContinuousOperations",
    "annualEarningsFromEquityInterestNetOfTax",
    "annualTaxProvision",
    "annualPretaxIncome",
    "annualOtherIncomeExpense",
    "annualOtherNonOperatingIncomeExpenses",
    "annualSpecialIncomeCharges",
    "annualGainOnSaleOfPPE",
    "annualGainOnSaleOfBusiness",
    "annualOtherSpecialCharges",
    "annualWriteOff",
    "annualImpairmentOfCapitalAssets",
    "annualRestructuringAndMergernAcquisition",
    "annualSecuritiesAmortization",
    "annualEarningsFromEquityInterest",
    "annualGainOnSaleOfSecurity",
    "annualNetNonOperatingInterestIncomeExpense",
    "annualTotalOtherFinanceCost",
    "annualInterestExpenseNonOperating",
    "annualInterestIncomeNonOperating",
    "annualOperatingIncome",
    "annualOperatingExpense",
    "annualOtherOperatingExpenses",
    "annualOtherTaxes",
    "annualProvisionForDoubtfulAccounts",
    "annualDepreciationAmortizationDepletionIncomeStatement",
    "annualDepletionIncomeStatement",
    "annualDepreciationAndAmortizationInIncomeStatement",
    "annualAmortization",
    "annualAmortizationOfIntangiblesIncomeStatement",
    "annualDepreciationIncomeStatement",
    "annualResearchAndDevelopment",
    "annualSellingGeneralAndAdministration",
    "annualSellingAndMarketingExpense",
    "annualGeneralAndAdministrativeExpense",
    "annualOtherGandA",
    "annualInsuranceAndClaims",
    "annualRentAndLandingFees",
    "annualSalariesAndWages",
    "annualGrossProfit",
    "annualCostOfRevenue",
    "annualTotalRevenue",
    "annualExciseTaxes",
    "annualOperatingRevenue",
]

ANNUAL_CASH_FLOW_COLUMNS = [
    "annualForeignSales",
    "annualDomesticSales",
    "annualAdjustedGeographySegmentData",
    "annualFreeCashFlow",
    "annualRepurchaseOfCapitalStock",
    "annualRepaymentOfDebt",
    "annualIssuanceOfDebt",
    "annualIssuanceOfCapitalStock",
    "annualCapitalExpenditure",
    "annualInterestPaidSupplementalData",
    "annualIncomeTaxPaidSupplementalData",
    "annualEndCashPosition",
    "annualOtherCashAdjustmentOutsideChangeinCash",
    "annualBeginningCashPosition",
    "annualEffectOfExchangeRateChanges",
    "annualChangesInCash",
    "annualOtherCashAdjustmentInsideChangeinCash",
    "annualCashFlowFromDiscontinuedOperation",
    "annualFinancingCashFlow",
    "annualCashFromDiscontinuedFinancingActivities",
    "annualCashFlowFromContinuingFinancingActivities",
    "annualNetOtherFinancingCharges",
    "annualInterestPaidCFF",
    "annualProceedsFromStockOptionExercised",
    "annualCashDividendsPaid",
    "annualPreferredStockDividendPaid",
    "annualCommonStockDividendPaid",
    "annualNetPreferredStockIssuance",
    "annualPreferredStockPayments",
    "annualPreferredStockIssuance",
    "annualNetCommonStockIssuance",
    "annualCommonStockPayments",
    "annualCommonStockIssuance",
    "annualNetIssuancePaymentsOfDebt",
    "annualNetShortTermDebtIssuance",
    "annualShortTermDebtPayments",
    "annualShortTermDebtIssuance",
    "annualNetLongTermDebtIssuance",
    "annualLongTermDebtPayments",
    "annualLongTermDebtIssuance",
    "annualInvestingCashFlow",
    "annualCashFromDiscontinuedInvestingActivities",
    "annualCashFlowFromContinuingInvestingActivities",
    "annualNetOtherInvestingChanges",
    "annualInterestReceivedCFI",
    "annualDividendsReceivedCFI",
    "annualNetInvestmentPurchaseAndSale",
    "annualSaleOfInvestment",
    "annualPurchaseOfInvestment",
    "annualNetInvestmentPropertiesPurchaseAndSale",
    "annualSaleOfInvestmentProperties",
    "annualPurchaseOfInvestmentProperties",
    "annualNetBusinessPurchaseAndSale",
    "annualSaleOfBusiness",
    "annualPurchaseOfBusiness",
    "annualNetIntangiblesPurchaseAndSale",
    "annualSaleOfIntangibles",
    "annualPurchaseOfIntangibles",
    "annualNetPPEPurchaseAndSale",
    "annualSaleOfPPE",
    "annualPurchaseOfPPE",
    "annualCapitalExpenditureReported",
    "annualOperatingCashFlow",
    "annualCashFromDiscontinuedOperatingActivities",
    "annualCashFlowFromContinuingOperatingActivities",
    "annualTaxesRefundPaid",
    "annualInterestReceivedCFO",
    "annualInterestPaidCFO",
    "annualDividendReceivedCFO",
    "annualDividendPaidCFO",
    "annualChangeInWorkingCapital",
    "annualChangeInOtherWorkingCapital",
    "annualChangeInOtherCurrentLiabilities",
    "annualChangeInOtherCurrentAssets",
    "annualChangeInPayablesAndAccruedExpense",
    "annualChangeInAccruedExpense",
    "annualChangeInInterestPayable",
    "annualChangeInPayable",
    "annualChangeInDividendPayable",
    "annualChangeInAccountPayable",
    "annualChangeInTaxPayable",
    "annualChangeInIncomeTaxPayable",
    "annualChangeInPrepaidAssets",
    "annualChangeInInventory",
    "annualChangeInReceivables",
    "annualChangesInAccountReceivables",
    "annualOtherNonCashItems",
    "annualExcessTaxBenefitFromStockBasedCompensation",
    "annualStockBasedCompensation",
    "annualUnrealizedGainLossOnInvestmentSecurities",
    "annualProvisionandWriteOffofAssets",
    "annualAssetImpairmentCharge",
    "annualAmortizationOfSecurities",
    "annualDeferredTax",
    "annualDeferredIncomeTax",
    "annualDepreciationAmortizationDepletion",
    "annualDepletion",
    "annualDepreciationAndAmortization",
    "annualAmortizationCashFlow",
    "annualAmortizationOfIntangibles",
    "annualDepreciation",
    "annualOperatingGainsLosses",
    "annualPensionAndEmployeeBenefitExpense",
    "annualEarningsLossesFromEquityInvestments",
    "annualGainLossOnInvestmentSecurities",
    "annualNetForeignCurrencyExchangeGainLoss",
    "annualGainLossOnSaleOfPPE",
    "annualGainLossOnSaleOfBusiness",
    "annualNetIncomeFromContinuingOperations",
    "annualCashFlowsfromusedinOperatingActivitiesDirect",
    "annualTaxesRefundPaidDirect",
    "annualInterestReceivedDirect",
    "annualInterestPaidDirect",
    "annualDividendsReceivedDirect",
    "annualDividendsPaidDirect",
    "annualClassesofCashPayments",
    "annualOtherCashPaymentsfromOperatingActivities",
    "annualPaymentsonBehalfofEmployees",
    "annualPaymentstoSuppliersforGoodsandServices",
    "annualClassesofCashReceiptsfromOperatingActivities",
    "annualOtherCashReceiptsfromOperatingActivities",
    "annualReceiptsfromGovernmentGrants",
    "annualReceiptsfromCustomers",
]

QUARTERLY_CASH_FLOW_COLUMNS = [
    "quarterlyForeignSales",
    "quarterlyDomesticSales",
    "quarterlyAdjustedGeographySegmentData",
    "quarterlyFreeCashFlow",
    "quarterlyRepurchaseOfCapitalStock",
    "quarterlyRepaymentOfDebt",
    "quarterlyIssuanceOfDebt",
    "quarterlyIssuanceOfCapitalStock",
    "quarterlyCapitalExpenditure",
    "quarterlyInterestPaidSupplementalData",
    "quarterlyIncomeTaxPaidSupplementalData",
    "quarterlyEndCashPosition",
    "quarterlyOtherCashAdjustmentOutsideChangeinCash",
    "quarterlyBeginningCashPosition",
    "quarterlyEffectOfExchangeRateChanges",
    "quarterlyChangesInCash",
    "quarterlyOtherCashAdjustmentInsideChangeinCash",
    "quarterlyCashFlowFromDiscontinuedOperation",
    "quarterlyFinancingCashFlow",
    "quarterlyCashFromDiscontinuedFinancingActivities",
    "quarterlyCashFlowFromContinuingFinancingActivities",
    "quarterlyNetOtherFinancingCharges",
    "quarterlyInterestPaidCFF",
    "quarterlyProceedsFromStockOptionExercised",
    "quarterlyCashDividendsPaid",
    "quarterlyPreferredStockDividendPaid",
    "quarterlyCommonStockDividendPaid",
    "quarterlyNetPreferredStockIssuance",
    "quarterlyPreferredStockPayments",
    "quarterlyPreferredStockIssuance",
    "quarterlyNetCommonStockIssuance",
    "quarterlyCommonStockPayments",
    "quarterlyCommonStockIssuance",
    "quarterlyNetIssuancePaymentsOfDebt",
    "quarterlyNetShortTermDebtIssuance",
    "quarterlyShortTermDebtPayments",
    "quarterlyShortTermDebtIssuance",
    "quarterlyNetLongTermDebtIssuance",
    "quarterlyLongTermDebtPayments",
    "quarterlyLongTermDebtIssuance",
    "quarterlyInvestingCashFlow",
    "quarterlyCashFromDiscontinuedInvestingActivities",
    "quarterlyCashFlowFromContinuingInvestingActivities",
    "quarterlyNetOtherInvestingChanges",
    "quarterlyInterestReceivedCFI",
    "quarterlyDividendsReceivedCFI",
    "quarterlyNetInvestmentPurchaseAndSale",
    "quarterlySaleOfInvestment",
    "quarterlyPurchaseOfInvestment",
    "quarterlyNetInvestmentPropertiesPurchaseAndSale",
    "quarterlySaleOfInvestmentProperties",
    "quarterlyPurchaseOfInvestmentProperties",
    "quarterlyNetBusinessPurchaseAndSale",
    "quarterlySaleOfBusiness",
    "quarterlyPurchaseOfBusiness",
    "quarterlyNetIntangiblesPurchaseAndSale",
    "quarterlySaleOfIntangibles",
    "quarterlyPurchaseOfIntangibles",
    "quarterlyNetPPEPurchaseAndSale",
    "quarterlySaleOfPPE",
    "quarterlyPurchaseOfPPE",
    "quarterlyCapitalExpenditureReported",
    "quarterlyOperatingCashFlow",
    "quarterlyCashFromDiscontinuedOperatingActivities",
    "quarterlyCashFlowFromContinuingOperatingActivities",
    "quarterlyTaxesRefundPaid",
    "quarterlyInterestReceivedCFO",
    "quarterlyInterestPaidCFO",
    "quarterlyDividendReceivedCFO",
    "quarterlyDividendPaidCFO",
    "quarterlyChangeInWorkingCapital",
    "quarterlyChangeInOtherWorkingCapital",
    "quarterlyChangeInOtherCurrentLiabilities",
    "quarterlyChangeInOtherCurrentAssets",
    "quarterlyChangeInPayablesAndAccruedExpense",
    "quarterlyChangeInAccruedExpense",
    "quarterlyChangeInInterestPayable",
    "quarterlyChangeInPayable",
    "quarterlyChangeInDividendPayable",
    "quarterlyChangeInAccountPayable",
    "quarterlyChangeInTaxPayable",
    "quarterlyChangeInIncomeTaxPayable",
    "quarterlyChangeInPrepaidAssets",
    "quarterlyChangeInInventory",
    "quarterlyChangeInReceivables",
    "quarterlyChangesInAccountReceivables",
    "quarterlyOtherNonCashItems",
    "quarterlyExcessTaxBenefitFromStockBasedCompensation",
    "quarterlyStockBasedCompensation",
    "quarterlyUnrealizedGainLossOnInvestmentSecurities",
    "quarterlyProvisionandWriteOffofAssets",
    "quarterlyAssetImpairmentCharge",
    "quarterlyAmortizationOfSecurities",
    "quarterlyDeferredTax",
    "quarterlyDeferredIncomeTax",
    "quarterlyDepreciationAmortizationDepletion",
    "quarterlyDepletion",
    "quarterlyDepreciationAndAmortization",
    "quarterlyAmortizationCashFlow",
    "quarterlyAmortizationOfIntangibles",
    "quarterlyDepreciation",
    "quarterlyOperatingGainsLosses",
    "quarterlyPensionAndEmployeeBenefitExpense",
    "quarterlyEarningsLossesFromEquityInvestments",
    "quarterlyGainLossOnInvestmentSecurities",
    "quarterlyNetForeignCurrencyExchangeGainLoss",
    "quarterlyGainLossOnSaleOfPPE",
    "quarterlyGainLossOnSaleOfBusiness",
    "quarterlyNetIncomeFromContinuingOperations",
    "quarterlyCashFlowsfromusedinOperatingActivitiesDirect",
    "quarterlyTaxesRefundPaidDirect",
    "quarterlyInterestReceivedDirect",
    "quarterlyInterestPaidDirect",
    "quarterlyDividendsReceivedDirect",
    "quarterlyDividendsPaidDirect",
    "quarterlyClassesofCashPayments",
    "quarterlyOtherCashPaymentsfromOperatingActivities",
    "quarterlyPaymentsonBehalfofEmployees",
    "quarterlyPaymentstoSuppliersforGoodsandServices",
    "quarterlyClassesofCashReceiptsfromOperatingActivities",
    "quarterlyOtherCashReceiptsfromOperatingActivities",
    "quarterlyReceiptsfromGovernmentGrants",
    "quarterlyReceiptsfromCustomer",
]

ANNUAL_BALANCE_SHEET_COLUMNS = [
    "annualTreasurySharesNumber",
    "annualPreferredSharesNumber",
    "annualOrdinarySharesNumber",
    "annualShareIssued",
    "annualNetDebt",
    "annualTotalDebt",
    "annualTangibleBookValue",
    "annualInvestedCapital",
    "annualWorkingCapital",
    "annualNetTangibleAssets",
    "annualCapitalLeaseObligations",
    "annualCommonStockEquity",
    "annualPreferredStockEquity",
    "annualTotalCapitalization",
    "annualTotalEquityGrossMinorityInterest",
    "annualMinorityInterest",
    "annualStockholdersEquity",
    "annualOtherEquityInterest",
    "annualGainsLossesNotAffectingRetainedEarnings",
    "annualOtherEquityAdjustments",
    "annualFixedAssetsRevaluationReserve",
    "annualForeignCurrencyTranslationAdjustments",
    "annualMinimumPensionLiabilities",
    "annualUnrealizedGainLoss",
    "annualTreasuryStock",
    "annualRetainedEarnings",
    "annualAdditionalPaidInCapital",
    "annualCapitalStock",
    "annualOtherCapitalStock",
    "annualCommonStock",
    "annualPreferredStock",
    "annualTotalPartnershipCapital",
    "annualGeneralPartnershipCapital",
    "annualLimitedPartnershipCapital",
    "annualTotalLiabilitiesNetMinorityInterest",
    "annualTotalNonCurrentLiabilitiesNetMinorityInterest",
    "annualOtherNonCurrentLiabilities",
    "annualLiabilitiesHeldforSaleNonCurrent",
    "annualRestrictedCommonStock",
    "annualPreferredSecuritiesOutsideStockEquity",
    "annualDerivativeProductLiabilities",
    "annualEmployeeBenefits",
    "annualNonCurrentPensionAndOtherPostretirementBenefitPlans",
    "annualNonCurrentAccruedExpenses",
    "annualDuetoRelatedPartiesNonCurrent",
    "annualTradeandOtherPayablesNonCurrent",
    "annualNonCurrentDeferredLiabilities",
    "annualNonCurrentDeferredRevenue",
    "annualNonCurrentDeferredTaxesLiabilities",
    "annualLongTermDebtAndCapitalLeaseObligation",
    "annualLongTermCapitalLeaseObligation",
    "annualLongTermDebt",
    "annualLongTermProvisions",
    "annualCurrentLiabilities",
    "annualOtherCurrentLiabilities",
    "annualCurrentDeferredLiabilities",
    "annualCurrentDeferredRevenue",
    "annualCurrentDeferredTaxesLiabilities",
    "annualCurrentDebtAndCapitalLeaseObligation",
    "annualCurrentCapitalLeaseObligation",
    "annualCurrentDebt",
    "annualOtherCurrentBorrowings",
    "annualLineOfCredit",
    "annualCommercialPaper",
    "annualCurrentNotesPayable",
    "annualPensionandOtherPostRetirementBenefitPlansCurrent",
    "annualCurrentProvisions",
    "annualPayablesAndAccruedExpenses",
    "annualCurrentAccruedExpenses",
    "annualInterestPayable",
    "annualPayables",
    "annualOtherPayable",
    "annualDuetoRelatedPartiesCurrent",
    "annualDividendsPayable",
    "annualTotalTaxPayable",
    "annualIncomeTaxPayable",
    "annualAccountsPayable",
    "annualTotalAssets",
    "annualTotalNonCurrentAssets",
    "annualOtherNonCurrentAssets",
    "annualDefinedPensionBenefit",
    "annualNonCurrentPrepaidAssets",
    "annualNonCurrentDeferredAssets",
    "annualNonCurrentDeferredTaxesAssets",
    "annualDuefromRelatedPartiesNonCurrent",
    "annualNonCurrentNoteReceivables",
    "annualNonCurrentAccountsReceivable",
    "annualFinancialAssets",
    "annualInvestmentsAndAdvances",
    "annualOtherInvestments",
    "annualInvestmentinFinancialAssets",
    "annualHeldToMaturitySecurities",
    "annualAvailableForSaleSecurities",
    "annualFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
    "annualTradingSecurities",
    "annualLongTermEquityInvestment",
    "annualInvestmentsinJointVenturesatCost",
    "annualInvestmentsInOtherVenturesUnderEquityMethod",
    "annualInvestmentsinAssociatesatCost",
    "annualInvestmentsinSubsidiariesatCost",
    "annualInvestmentProperties",
    "annualGoodwillAndOtherIntangibleAssets",
    "annualOtherIntangibleAssets",
    "annualGoodwill",
    "annualNetPPE",
    "annualAccumulatedDepreciation",
    "annualGrossPPE",
    "annualLeases",
    "annualConstructionInProgress",
    "annualOtherProperties",
    "annualMachineryFurnitureEquipment",
    "annualBuildingsAndImprovements",
    "annualLandAndImprovements",
    "annualProperties",
    "annualCurrentAssets",
    "annualOtherCurrentAssets",
    "annualHedgingAssetsCurrent",
    "annualAssetsHeldForSaleCurrent",
    "annualCurrentDeferredAssets",
    "annualCurrentDeferredTaxesAssets",
    "annualRestrictedCash",
    "annualPrepaidAssets",
    "annualInventory",
    "annualInventoriesAdjustmentsAllowances",
    "annualOtherInventories",
    "annualFinishedGoods",
    "annualWorkInProcess",
    "annualRawMaterials",
    "annualReceivables",
    "annualReceivablesAdjustmentsAllowances",
    "annualOtherReceivables",
    "annualDuefromRelatedPartiesCurrent",
    "annualTaxesReceivable",
    "annualAccruedInterestReceivable",
    "annualNotesReceivable",
    "annualLoansReceivable",
    "annualAccountsReceivable",
    "annualAllowanceForDoubtfulAccountsReceivable",
    "annualGrossAccountsReceivable",
    "annualCashCashEquivalentsAndShortTermInvestments",
    "annualOtherShortTermInvestments",
    "annualCashAndCashEquivalents",
    "annualCashEquivalents",
    "annualCashFinancial",
]

QUARTERLY_BALANCE_SHEET_COLUMNS = [
    "quarterlyTreasurySharesNumber",
    "quarterlyPreferredSharesNumber",
    "quarterlyOrdinarySharesNumber",
    "quarterlyShareIssued",
    "quarterlyNetDebt",
    "quarterlyTotalDebt",
    "quarterlyTangibleBookValue",
    "quarterlyInvestedCapital",
    "quarterlyWorkingCapital",
    "quarterlyNetTangibleAssets",
    "quarterlyCapitalLeaseObligations",
    "quarterlyCommonStockEquity",
    "quarterlyPreferredStockEquity",
    "quarterlyTotalCapitalization",
    "quarterlyTotalEquityGrossMinorityInterest",
    "quarterlyMinorityInterest",
    "quarterlyStockholdersEquity",
    "quarterlyOtherEquityInterest",
    "quarterlyGainsLossesNotAffectingRetainedEarnings",
    "quarterlyOtherEquityAdjustments",
    "quarterlyFixedAssetsRevaluationReserve",
    "quarterlyForeignCurrencyTranslationAdjustments",
    "quarterlyMinimumPensionLiabilities",
    "quarterlyUnrealizedGainLoss",
    "quarterlyTreasuryStock",
    "quarterlyRetainedEarnings",
    "quarterlyAdditionalPaidInCapital",
    "quarterlyCapitalStock",
    "quarterlyOtherCapitalStock",
    "quarterlyCommonStock",
    "quarterlyPreferredStock",
    "quarterlyTotalPartnershipCapital",
    "quarterlyGeneralPartnershipCapital",
    "quarterlyLimitedPartnershipCapital",
    "quarterlyTotalLiabilitiesNetMinorityInterest",
    "quarterlyTotalNonCurrentLiabilitiesNetMinorityInterest",
    "quarterlyOtherNonCurrentLiabilities",
    "quarterlyLiabilitiesHeldforSaleNonCurrent",
    "quarterlyRestrictedCommonStock",
    "quarterlyPreferredSecuritiesOutsideStockEquity",
    "quarterlyDerivativeProductLiabilities",
    "quarterlyEmployeeBenefits",
    "quarterlyNonCurrentPensionAndOtherPostretirementBenefitPlans",
    "quarterlyNonCurrentAccruedExpenses",
    "quarterlyDuetoRelatedPartiesNonCurrent",
    "quarterlyTradeandOtherPayablesNonCurrent",
    "quarterlyNonCurrentDeferredLiabilities",
    "quarterlyNonCurrentDeferredRevenue",
    "quarterlyNonCurrentDeferredTaxesLiabilities",
    "quarterlyLongTermDebtAndCapitalLeaseObligation",
    "quarterlyLongTermCapitalLeaseObligation",
    "quarterlyLongTermDebt",
    "quarterlyLongTermProvisions",
    "quarterlyCurrentLiabilities",
    "quarterlyOtherCurrentLiabilities",
    "quarterlyCurrentDeferredLiabilities",
    "quarterlyCurrentDeferredRevenue",
    "quarterlyCurrentDeferredTaxesLiabilities",
    "quarterlyCurrentDebtAndCapitalLeaseObligation",
    "quarterlyCurrentCapitalLeaseObligation",
    "quarterlyCurrentDebt",
    "quarterlyOtherCurrentBorrowings",
    "quarterlyLineOfCredit",
    "quarterlyCommercialPaper",
    "quarterlyCurrentNotesPayable",
    "quarterlyPensionandOtherPostRetirementBenefitPlansCurrent",
    "quarterlyCurrentProvisions",
    "quarterlyPayablesAndAccruedExpenses",
    "quarterlyCurrentAccruedExpenses",
    "quarterlyInterestPayable",
    "quarterlyPayables",
    "quarterlyOtherPayable",
    "quarterlyDuetoRelatedPartiesCurrent",
    "quarterlyDividendsPayable",
    "quarterlyTotalTaxPayable",
    "quarterlyIncomeTaxPayable",
    "quarterlyAccountsPayable",
    "quarterlyTotalAssets",
    "quarterlyTotalNonCurrentAssets",
    "quarterlyOtherNonCurrentAssets",
    "quarterlyDefinedPensionBenefit",
    "quarterlyNonCurrentPrepaidAssets",
    "quarterlyNonCurrentDeferredAssets",
    "quarterlyNonCurrentDeferredTaxesAssets",
    "quarterlyDuefromRelatedPartiesNonCurrent",
    "quarterlyNonCurrentNoteReceivables",
    "quarterlyNonCurrentAccountsReceivable",
    "quarterlyFinancialAssets",
    "quarterlyInvestmentsAndAdvances",
    "quarterlyOtherInvestments",
    "quarterlyInvestmentinFinancialAssets",
    "quarterlyHeldToMaturitySecurities",
    "quarterlyAvailableForSaleSecurities",
    "quarterlyFinancialAssetsDesignatedasFairValueThroughProfitorLossTotal",
    "quarterlyTradingSecurities",
    "quarterlyLongTermEquityInvestment",
    "quarterlyInvestmentsinJointVenturesatCost",
    "quarterlyInvestmentsInOtherVenturesUnderEquityMethod",
    "quarterlyInvestmentsinAssociatesatCost",
    "quarterlyInvestmentsinSubsidiariesatCost",
    "quarterlyInvestmentProperties",
    "quarterlyGoodwillAndOtherIntangibleAssets",
    "quarterlyOtherIntangibleAssets",
    "quarterlyGoodwill",
    "quarterlyNetPPE",
    "quarterlyAccumulatedDepreciation",
    "quarterlyGrossPPE",
    "quarterlyLeases",
    "quarterlyConstructionInProgress",
    "quarterlyOtherProperties",
    "quarterlyMachineryFurnitureEquipment",
    "quarterlyBuildingsAndImprovements",
    "quarterlyLandAndImprovements",
    "quarterlyProperties",
    "quarterlyCurrentAssets",
    "quarterlyOtherCurrentAssets",
    "quarterlyHedgingAssetsCurrent",
    "quarterlyAssetsHeldForSaleCurrent",
    "quarterlyCurrentDeferredAssets",
    "quarterlyCurrentDeferredTaxesAssets",
    "quarterlyRestrictedCash",
    "quarterlyPrepaidAssets",
    "quarterlyInventory",
    "quarterlyInventoriesAdjustmentsAllowances",
    "quarterlyOtherInventories",
    "quarterlyFinishedGoods",
    "quarterlyWorkInProcess",
    "quarterlyRawMaterials",
    "quarterlyReceivables",
    "quarterlyReceivablesAdjustmentsAllowances",
    "quarterlyOtherReceivables",
    "quarterlyDuefromRelatedPartiesCurrent",
    "quarterlyTaxesReceivable",
    "quarterlyAccruedInterestReceivable",
    "quarterlyNotesReceivable",
    "quarterlyLoansReceivable",
    "quarterlyAccountsReceivable",
    "quarterlyAllowanceForDoubtfulAccountsReceivable",
    "quarterlyGrossAccountsReceivable",
    "quarterlyCashCashEquivalentsAndShortTermInvestments",
    "quarterlyOtherShortTermInvestments",
    "quarterlyCashAndCashEquivalents",
    "quarterlyCashEquivalents",
    "quarterlyCashFinancial",
]

ANNUAL_FINANCIALS_COLUMNS = [
    "annualTaxEffectOfUnusualItems",
    "annualTaxRateForCalcs",
    "annualNormalizedEBITDA",
    "annualNormalizedDilutedEPS",
    "annualNormalizedBasicEPS",
    "annualTotalUnusualItems",
    "annualTotalUnusualItemsExcludingGoodwill",
    "annualNetIncomeFromContinuingOperationNetMinorityInterest",
    "annualReconciledDepreciation",
    "annualReconciledCostOfRevenue",
    "annualEBITDA",
    "annualEBIT",
    "annualNetInterestIncome",
    "annualInterestExpense",
    "annualInterestIncome",
    "annualContinuingAndDiscontinuedDilutedEPS",
    "annualContinuingAndDiscontinuedBasicEPS",
    "annualNormalizedIncome",
    "annualNetIncomeFromContinuingAndDiscontinuedOperation",
    "annualTotalExpenses",
    "annualRentExpenseSupplemental",
    "annualReportedNormalizedDilutedEPS",
    "annualReportedNormalizedBasicEPS",
    "annualTotalOperatingIncomeAsReported",
    "annualDividendPerShare",
    "annualDilutedAverageShares",
    "annualBasicAverageShares",
    "annualDilutedEPS",
    "annualDilutedEPSOtherGainsLosses",
    "annualTaxLossCarryforwardDilutedEPS",
    "annualDilutedAccountingChange",
    "annualDilutedExtraordinary",
    "annualDilutedDiscontinuousOperations",
    "annualDilutedContinuousOperations",
    "annualBasicEPS",
    "annualBasicEPSOtherGainsLosses",
    "annualTaxLossCarryforwardBasicEPS",
    "annualBasicAccountingChange",
    "annualBasicExtraordinary",
    "annualBasicDiscontinuousOperations",
    "annualBasicContinuousOperations",
    "annualDilutedNIAvailtoComStockholders",
    "annualAverageDilutionEarnings",
    "annualNetIncomeCommonStockholders",
    "annualOtherunderPreferredStockDividend",
    "annualPreferredStockDividends",
    "annualNetIncome",
    "annualMinorityInterests",
    "annualNetIncomeIncludingNoncontrollingInterests",
    "annualNetIncomeFromTaxLossCarryforward",
    "annualNetIncomeExtraordinary",
    "annualNetIncomeDiscontinuousOperations",
    "annualNetIncomeContinuousOperations",
    "annualEarningsFromEquityInterestNetOfTax",
    "annualTaxProvision",
    "annualPretaxIncome",
    "annualOtherIncomeExpense",
    "annualOtherNonOperatingIncomeExpenses",
    "annualSpecialIncomeCharges",
    "annualGainOnSaleOfPPE",
    "annualGainOnSaleOfBusiness",
    "annualOtherSpecialCharges",
    "annualWriteOff",
    "annualImpairmentOfCapitalAssets",
    "annualRestructuringAndMergernAcquisition",
    "annualSecuritiesAmortization",
    "annualEarningsFromEquityInterest",
    "annualGainOnSaleOfSecurity",
    "annualNetNonOperatingInterestIncomeExpense",
    "annualTotalOtherFinanceCost",
    "annualInterestExpenseNonOperating",
    "annualInterestIncomeNonOperating",
    "annualOperatingIncome",
    "annualOperatingExpense",
    "annualOtherOperatingExpenses",
    "annualOtherTaxes",
    "annualProvisionForDoubtfulAccounts",
    "annualDepreciationAmortizationDepletionIncomeStatement",
    "annualDepletionIncomeStatement",
    "annualDepreciationAndAmortizationInIncomeStatement",
    "annualAmortization",
    "annualAmortizationOfIntangiblesIncomeStatement",
    "annualDepreciationIncomeStatement",
    "annualResearchAndDevelopment",
    "annualSellingGeneralAndAdministration",
    "annualSellingAndMarketingExpense",
    "annualGeneralAndAdministrativeExpense",
    "annualOtherGandA",
    "annualInsuranceAndClaims",
    "annualRentAndLandingFees",
    "annualSalariesAndWages",
    "annualGrossProfit",
    "annualCostOfRevenue",
    "annualTotalRevenue",
    "annualExciseTaxes",
    "annualOperatingRevenue",
]

QUARTERLY_FINANCIALS_COLUMNS = [
    "quarterlyTaxEffectOfUnusualItems",
    "quarterlyTaxRateForCalcs",
    "quarterlyNormalizedEBITDA",
    "quarterlyNormalizedDilutedEPS",
    "quarterlyNormalizedBasicEPS",
    "quarterlyTotalUnusualItems",
    "quarterlyTotalUnusualItemsExcludingGoodwill",
    "quarterlyNetIncomeFromContinuingOperationNetMinorityInterest",
    "quarterlyReconciledDepreciation",
    "quarterlyReconciledCostOfRevenue",
    "quarterlyEBITDA",
    "quarterlyEBIT",
    "quarterlyNetInterestIncome",
    "quarterlyInterestExpense",
    "quarterlyInterestIncome",
    "quarterlyContinuingAndDiscontinuedDilutedEPS",
    "quarterlyContinuingAndDiscontinuedBasicEPS",
    "quarterlyNormalizedIncome",
    "quarterlyNetIncomeFromContinuingAndDiscontinuedOperation",
    "quarterlyTotalExpenses",
    "quarterlyRentExpenseSupplemental",
    "quarterlyReportedNormalizedDilutedEPS",
    "quarterlyReportedNormalizedBasicEPS",
    "quarterlyTotalOperatingIncomeAsReported",
    "quarterlyDividendPerShare",
    "quarterlyDilutedAverageShares",
    "quarterlyBasicAverageShares",
    "quarterlyDilutedEPS",
    "quarterlyDilutedEPSOtherGainsLosses",
    "quarterlyTaxLossCarryforwardDilutedEPS",
    "quarterlyDilutedAccountingChange",
    "quarterlyDilutedExtraordinary",
    "quarterlyDilutedDiscontinuousOperations",
    "quarterlyDilutedContinuousOperations",
    "quarterlyBasicEPS",
    "quarterlyBasicEPSOtherGainsLosses",
    "quarterlyTaxLossCarryforwardBasicEPS",
    "quarterlyBasicAccountingChange",
    "quarterlyBasicExtraordinary",
    "quarterlyBasicDiscontinuousOperations",
    "quarterlyBasicContinuousOperations",
    "quarterlyDilutedNIAvailtoComStockholders",
    "quarterlyAverageDilutionEarnings",
    "quarterlyNetIncomeCommonStockholders",
    "quarterlyOtherunderPreferredStockDividend",
    "quarterlyPreferredStockDividends",
    "quarterlyNetIncome",
    "quarterlyMinorityInterests",
    "quarterlyNetIncomeIncludingNoncontrollingInterests",
    "quarterlyNetIncomeFromTaxLossCarryforward",
    "quarterlyNetIncomeExtraordinary",
    "quarterlyNetIncomeDiscontinuousOperations",
    "quarterlyNetIncomeContinuousOperations",
    "quarterlyEarningsFromEquityInterestNetOfTax",
    "quarterlyTaxProvision",
    "quarterlyPretaxIncome",
    "quarterlyOtherIncomeExpense",
    "quarterlyOtherNonOperatingIncomeExpenses",
    "quarterlySpecialIncomeCharges",
    "quarterlyGainOnSaleOfPPE",
    "quarterlyGainOnSaleOfBusiness",
    "quarterlyOtherSpecialCharges",
    "quarterlyWriteOff",
    "quarterlyImpairmentOfCapitalAssets",
    "quarterlyRestructuringAndMergernAcquisition",
    "quarterlySecuritiesAmortization",
    "quarterlyEarningsFromEquityInterest",
    "quarterlyGainOnSaleOfSecurity",
    "quarterlyNetNonOperatingInterestIncomeExpense",
    "quarterlyTotalOtherFinanceCost",
    "quarterlyInterestExpenseNonOperating",
    "quarterlyInterestIncomeNonOperating",
    "quarterlyOperatingIncome",
    "quarterlyOperatingExpense",
    "quarterlyOtherOperatingExpenses",
    "quarterlyOtherTaxes",
    "quarterlyProvisionForDoubtfulAccounts",
    "quarterlyDepreciationAmortizationDepletionIncomeStatement",
    "quarterlyDepletionIncomeStatement",
    "quarterlyDepreciationAndAmortizationInIncomeStatement",
    "quarterlyAmortization",
    "quarterlyAmortizationOfIntangiblesIncomeStatement",
    "quarterlyDepreciationIncomeStatement",
    "quarterlyResearchAndDevelopment",
    "quarterlySellingGeneralAndAdministration",
    "quarterlySellingAndMarketingExpense",
    "quarterlyGeneralAndAdministrativeExpense",
    "quarterlyOtherGandA",
    "quarterlyInsuranceAndClaims",
    "quarterlyRentAndLandingFees",
    "quarterlySalariesAndWages",
    "quarterlyGrossProfit",
    "quarterlyCostOfRevenue",
    "quarterlyTotalRevenue",
    "quarterlyExciseTaxes",
    "quarterlyOperatingRevenue",
]



"""
Module to retrieve stock data from Yahoo Finance API
The main Ticker class inherits from this class
"""

from datetime import datetime
from typing import Literal

import pandas as pd

from stockdex import config
from stockdex.ticker_base import TickerBase


class TickerAPI(TickerBase):
    # current_timestamp = int(pd.Timestamp("now").timestamp())
    # five_years_ago = int(pd.Timestamp("now").timestamp()) - 5 * 365 * 24 * 60 * 60

    today = datetime.today()
    five_years_ago = today.replace(year=today.year - 5)

    def price(
        self,
        range: Literal[
            "1d", "5d", "1mo", "3mo", "6mo", "1y", "2y", "5y", "10y", "ytd", "max"
        ] = "1d",
        dataGranularity: Literal[
            "1m",
            "2m",
            "5m",
            "15m",
            "30m",
            "60m",
            "90m",
            "1h",
            "1d",
            "5d",
            "1wk",
            "1mo",
            "3mo",
        ] = "1m",
    ) -> pd.DataFrame:
        """
        Get the price data for the stock

        Args:
        range (str): The range of the price data to retrieve
        valid values are "1d", "5d", "1mo", "3mo", "6mo", "1y", "2y", "5y", "10y", "ytd", "max"

        dataGranularity (str): The granularity of the data to retrieve (interval)
        valid values are "1m", "2m", "5m", "15m", "30m", "60m", "90m", "1h", "1d", "5d", "1wk", "1mo", "3mo""  # noqa: E501
        """

        url = f"{config.BASE_URL}/chart/{self.ticker}?range={range}&interval={dataGranularity}"
        response = self.get_response(url)

        meta = response.json()["chart"]["result"][0]["meta"]
        currency = meta["currency"]
        exchangeTimezoneName = meta["exchangeTimezoneName"]
        timezone = meta["timezone"]
        exchangeName = meta["exchangeName"]
        instrumentType = meta["instrumentType"]

        timestamp = response.json()["chart"]["result"][0]["timestamp"]
        timestamp = pd.to_datetime(timestamp, unit="s")

        indicators = response.json()["chart"]["result"][0]["indicators"]
        volume = indicators["quote"][0]["volume"]
        close = indicators["quote"][0]["close"]
        open = indicators["quote"][0]["open"]
        high = indicators["quote"][0]["high"]
        low = indicators["quote"][0]["low"]

        return pd.DataFrame(
            {
                "timestamp": timestamp,
                "volume": volume,
                "close": close,
                "open": open,
                "high": high,
                "low": low,
                "currency": currency,
                "timezone": timezone,
                "exchangeTimezoneName": exchangeTimezoneName,
                "exchangeName": exchangeName,
                "instrumentType": instrumentType,
            }
        )

    @property
    def current_trading_period(self) -> pd.DataFrame:
        """
        Get the current trading period for the stock
        """

        url = f"{config.BASE_URL}/chart/{self.ticker}"
        response = self.get_response(url)

        currentTradingPeriod = response.json()["chart"]["result"][0]["meta"][
            "currentTradingPeriod"
        ]

        pre = currentTradingPeriod["pre"]
        regular = currentTradingPeriod["regular"]
        post = currentTradingPeriod["post"]

        # convert timestamps to datetime
        pre["start"] = pd.to_datetime(pre["start"], unit="s")
        pre["end"] = pd.to_datetime(pre["end"], unit="s")
        regular["start"] = pd.to_datetime(regular["start"], unit="s")
        regular["end"] = pd.to_datetime(regular["end"], unit="s")
        post["start"] = pd.to_datetime(post["start"], unit="s")
        post["end"] = pd.to_datetime(post["end"], unit="s")

        return pd.DataFrame(
            {
                "pre": pre,
                "regular": regular,
                "post": post,
            }
        )

    def income_statement(
        self,
        frequency: Literal["annual", "quarterly"] = "annual",
        format: Literal["fmt", "raw"] = "fmt",
        period1: datetime = five_years_ago,
        period2: datetime = today,
    ) -> pd.DataFrame:
        """
        Get the income statement for the stock

        Args:
        frequency (str): The frequency of the data to retrieve
        valid values are "annual", "quarterly"

        format (str): The format of the data to retrieve
        valid values are "fmt", "raw"
        if "fmt" is used, the data will be in a human readable format, e.g. 1B
        if "raw" is used, the data will be in a raw format, e.g. 1000000000

        period1 (datetime): The start date of the data to retrieve
        default is five years ago as that is the maximum period the API supports data retrieval for

        period2 (datetime): The end date of the data to retrieve
        default is the current date
        """
        url = self.build_url(frequency, period1, period2, "income_statement")

        response = self.get_response(url).json()["timeseries"]["result"]

        return self.extract_dataframe(response, format)

    def cash_flow(
        self,
        frequency: Literal["annual", "quarterly"] = "annual",
        format: Literal["fmt", "raw"] = "fmt",
        period1: datetime = five_years_ago,
        period2: datetime = today,
    ) -> pd.DataFrame:
        """
        Get the cash flow statement for the stock

        Args:
        frequency (str): The frequency of the data to retrieve
        valid values are "annual", "quarterly"

        format (str): The format of the data to retrieve
        valid values are "fmt", "raw"
        if "fmt" is used, the data will be in a human readable format, e.g. 1B
        if "raw" is used, the data will be in a raw format, e.g. 1000000000

        period1 (datetime): The start date of the data to retrieve
        default is five years ago as that is the maximum period the API supports data retrieval for

        period2 (datetime): The end date of the data to retrieve
        default is the current date
        """
        url = self.build_url(frequency, period1, period2, "cash_flow")

        response = self.get_response(url).json()["timeseries"]["result"]

        return self.extract_dataframe(response, format)

    def balance_sheet(
        self,
        frequency: Literal["annual", "quarterly"] = "annual",
        format: Literal["fmt", "raw"] = "fmt",
        period1: datetime = five_years_ago,
        period2: datetime = today,
    ) -> pd.DataFrame:
        """
        Get the balance sheet for the stock

        Args:
        frequency (str): The frequency of the data to retrieve
        valid values are "annual", "quarterly"

        format (str): The format of the data to retrieve
        valid values are "fmt", "raw"
        if "fmt" is used, the data will be in a human readable format, e.g. 1B
        if "raw" is used, the data will be in a raw format, e.g. 1000000000

        period1 (datetime): The start date of the data to retrieve
        default is five years ago as that is the maximum period the API supports data retrieval for

        period2 (datetime): The end date of the data to retrieve
        default is the current date
        """
        url = self.build_url(frequency, period1, period2, "balance_sheet")

        response = self.get_response(url).json()["timeseries"]["result"]

        return self.extract_dataframe(response, format)

    def financials(
        self,
        frequency: Literal["annual", "quarterly"] = "annual",
        format: Literal["fmt", "raw"] = "fmt",
        period1: datetime = five_years_ago,
        period2: datetime = today,
    ) -> pd.DataFrame:
        """
        Get the financials for the stock

        Args:
        frequency (str): The frequency of the data to retrieve
        valid values are "annual", "quarterly"

        format (str): The format of the data to retrieve
        valid values are "fmt", "raw"
        if "fmt" is used, the data will be in a human readable format, e.g. 1B
        if "raw" is used, the data will be in a raw format, e.g. 1000000000

        period1 (datetime): The start date of the data to retrieve
        default is five years ago as that is the maximum period the API supports data retrieval for

        period2 (datetime): The end date of the data to retrieve
        default is the current date
        """
        url = self.build_url(frequency, period1, period2, "financials")

        response = self.get_response(url).json()["timeseries"]["result"]

        return self.extract_dataframe(response, format)

    def build_url(
        self,
        frequency: str,
        period1: datetime,
        period2: datetime,
        desired_entity: str,
    ) -> str:
        """
        Build the URL for the income statement, balance sheet, and cash flow statement
        """
        # convert period1 and period2 to timestamps
        period1 = int(pd.Timestamp(period1).timestamp())
        period2 = int(pd.Timestamp(period2).timestamp())

        columns = ",".join(
            getattr(config, f"{frequency.upper()}_{desired_entity.upper()}_COLUMNS")
        )
        url = f"{config.FUNDAMENTALS_BASE_URL}/{self.ticker}/?symbol={self.ticker}"
        url += f"&type={columns}"
        url += f"&period1={period1}&period2={period2}"
        return url

    def extract_dataframe(self, response, format="fmt") -> pd.DataFrame:
        """
        Extract the dataframes from the response
        """
        row = {}
        for item in response:
            column = item["meta"]["type"][0]

            # skip if there is no data for in the response
            if column not in item:
                continue

            data = item[column]
            dated_index = [i["asOfDate"] for i in data]
            values = [i["reportedValue"][format] for i in data]

            row[column] = pd.Series(values, index=dated_index)

        return pd.DataFrame(row, index=dated_index)



"""
Base class for ticker objects to inherit from
"""

import requests

from stockdex.lib import get_user_agent


class TickerBase:
    request_headers = {
        "User-Agent": get_user_agent(),
    }

    def get_response(self, url: str) -> requests.Response:
        """
        Send an HTTP GET request to the website

        Args:
        url (str): The URL of the website
        headers (dict): The headers to be sent with the HTTP GET request

        Returns:
        requests.Response: The response of the HTTP GET request
        """

        # Send an HTTP GET request to the website
        session = requests.Session()
        response = session.get(url, headers=self.request_headers, timeout=2)
        # If the HTTP GET request can't be served
        if response.status_code != 200:
            raise Exception("Failed to load page, check if the ticker symbol exists")

        return response



"""
Module to test the NasdaqInterface class
"""

import pandas as pd
import pytest

from stockdex.nasdaq_interface import NASDAQInterface


@pytest.mark.skip(reason="Under development")
@pytest.mark.parametrize(
    "ticker",
    [
        ("NVDA"),
        ("AAPL"),
        ("GOOGL"),
        ("MSFT"),
    ],
)
def test_quarterly_earnings(ticker):
    nasdaq_interface = NASDAQInterface(ticker)
    response = nasdaq_interface.quarterly_earnings()

    assert response is not None
    assert isinstance(response, pd.DataFrame)
    assert response.shape[0] > 0



from datetime import datetime

import pandas as pd
import pytest

from stockdex.ticker import Ticker


@pytest.mark.parametrize(
    "ticker, range, dataGranularity",
    [
        ("AAPL", "1d", "1h"),
        ("GOOGL", "1d", "1h"),
        ("MSFT", "1d", "1d"),
        ("AAPL", "1h", "1m"),
        ("GOOGL", "5d", "1d"),
        ("MSFT", "1d", "1d"),
        ("NVDA", "max", "3mo"),
        ("SAP", "1y", "1mo"),
    ],
)
def test_price(ticker, range, dataGranularity):
    """
    Test the price property of the TickerAPI class
    """

    ticker = Ticker(ticker)
    price = ticker.price(range=range, dataGranularity=dataGranularity)

    assert price.columns.tolist() == [
        "timestamp",
        "volume",
        "close",
        "open",
        "high",
        "low",
        "currency",
        "timezone",
        "exchangeTimezoneName",
        "exchangeName",
        "instrumentType",
    ]
    assert len(price) > 0


@pytest.mark.parametrize(
    "ticker",
    [
        ("AAPL"),
        ("NVDA"),
        ("FMC"),
    ],
)
def test_current_trading_period(ticker):
    """
    Test the price property of the TickerAPI class
    """

    ticker = Ticker(ticker)
    current_trading_period = ticker.current_trading_period

    assert current_trading_period.columns.tolist() == [
        "pre",
        "regular",
        "post",
    ]
    assert len(current_trading_period) == 4
    assert current_trading_period.index.tolist() == [
        "timezone",
        "start",
        "end",
        "gmtoffset",
    ]


@pytest.mark.parametrize(
    "ticker, frequency, format, period1, period2",
    [
        ("AAPL", "quarterly", "fmt", datetime(2020, 1, 1), datetime.today()),
        ("GOOGL", "quarterly", "fmt", datetime(2021, 1, 1), datetime.today()),
        ("MSFT", "quarterly", "fmt", datetime(2023, 1, 1), datetime(2024, 1, 1)),
        ("NVDA", "quarterly", "raw", datetime(2020, 1, 1), datetime.today()),
        ("FMC", "quarterly", "raw", datetime(2021, 1, 1), datetime.today()),
    ],
)
def test_income_statement(ticker, frequency, format, period1, period2):
    ticker = Ticker(ticker)
    income_statement = ticker.income_statement(
        frequency=frequency, format=format, period1=period1, period2=period2
    )

    assert isinstance(income_statement, pd.DataFrame)
    assert income_statement.shape[0] > 0
    assert income_statement.shape[1] > 0


@pytest.mark.parametrize(
    "ticker, frequency, format, period1, period2",
    [
        ("AAPL", "quarterly", "fmt", datetime(2020, 1, 1), datetime.today()),
        ("GOOGL", "quarterly", "fmt", datetime(2021, 1, 1), datetime.today()),
        ("MSFT", "quarterly", "fmt", datetime(2023, 1, 1), datetime(2024, 1, 1)),
        ("NVDA", "quarterly", "raw", datetime(2020, 1, 1), datetime.today()),
        ("FMC", "quarterly", "raw", datetime(2021, 1, 1), datetime.today()),
    ],
)
def test_cash_flow(ticker, frequency, format, period1, period2):
    ticker = Ticker(ticker)
    cash_flow = ticker.cash_flow(
        frequency=frequency, format=format, period1=period1, period2=period2
    )

    assert isinstance(cash_flow, pd.DataFrame)
    assert cash_flow.shape[0] > 0
    assert cash_flow.shape[1] > 0


@pytest.mark.parametrize(
    "ticker, frequency, format, period1, period2",
    [
        ("AAPL", "quarterly", "fmt", datetime(2020, 1, 1), datetime.today()),
        ("GOOGL", "quarterly", "fmt", datetime(2021, 1, 1), datetime.today()),
        ("MSFT", "quarterly", "fmt", datetime(2023, 1, 1), datetime(2024, 1, 1)),
        ("NVDA", "quarterly", "raw", datetime(2020, 1, 1), datetime.today()),
        ("FMC", "quarterly", "raw", datetime(2021, 1, 1), datetime.today()),
    ],
)
def test_balance_sheet(ticker, frequency, format, period1, period2):
    ticker = Ticker(ticker)
    balance_sheet = ticker.balance_sheet(
        frequency=frequency, format=format, period1=period1, period2=period2
    )

    assert isinstance(balance_sheet, pd.DataFrame)
    assert balance_sheet.shape[0] > 0
    assert balance_sheet.shape[1] > 0


@pytest.mark.parametrize(
    "ticker, frequency, format, period1, period2",
    [
        ("AAPL", "quarterly", "fmt", datetime(2020, 1, 1), datetime.today()),
        ("GOOGL", "quarterly", "fmt", datetime(2021, 1, 1), datetime.today()),
        ("MSFT", "quarterly", "fmt", datetime(2023, 1, 1), datetime(2024, 1, 1)),
        ("NVDA", "quarterly", "raw", datetime(2020, 1, 1), datetime.today()),
        ("FMC", "quarterly", "raw", datetime(2021, 1, 1), datetime.today()),
    ],
)
def test_financials(ticker, frequency, format, period1, period2):
    ticker = Ticker(ticker)
    financials = ticker.financials(
        frequency=frequency, format=format, period1=period1, period2=period2
    )

    assert isinstance(financials, pd.DataFrame)
    assert financials.shape[0] > 0
    assert financials.shape[1] > 0



"""
Interface for NASDAQ stock data
"""

import requests
from bs4 import BeautifulSoup

from stockdex.lib import get_user_agent


class NASDAQInterface:
    def __init__(self, ticker):
        self.ticker = ticker
        self.base_url = "https://www.nasdaq.com/market-activity/stocks/"
        self.request_headers = {
            "User-Agent": get_user_agent()[0],
        }

    def get_response(self, url: str, headers: dict = None) -> requests.Response:
        """
        Send an HTTP GET request to the website

        Args:
        url (str): The URL of the website
        headers (dict): The headers to be sent with the HTTP GET request

        Returns:
        requests.Response: The response of the HTTP GET request
        """

        # Send an HTTP GET request to the website
        session = requests.Session()
        response = session.get(url, headers=headers)
        # If the HTTP GET request can't be served
        if response.status_code != 200:
            raise Exception("Failed to load page, check if the ticker symbol exists")

        return response

    def quarterly_earnings(self):
        """
        Get quarterly earnings for the stock
        """

        url = f"{self.base_url}/{self.ticker}/earnings"
        # send a get request to the website
        response = self.get_response(url, headers=self.request_headers)

        soup = BeautifulSoup(response.content, "html.parser")

        tables = soup.find_all("table")

        # TODO: Fix this as it does not return anything now
        headers = [header.content for header in tables[0].find_all("th")]
        data = [data.content for data in tables[0].find_all("td")]

        return headers, data
